@@ -712,6 +712,42 @@ QModelIndex KFilePlacesModel::closestItem(const QUrl &url) const
     }
 }
 
+QString KFilePlacesModel::placeCollapsedUrl(const QUrl &_url) const
+{
+    // Mimic KUrlNavigator and show a pretty place name,
+    // for example Documents/foo/bar rather than /home/user/Documents/foo/bar
+    const QModelIndex closestIdx = closestItem(_url);
+    if (!closestIdx.isValid()) {
+        return QString();
+    }
+
+    const QUrl placeUrl = url(closestIdx);
+
+    QString result = text(closestIdx);
+    // Guard again erroneous user-places.xbel file
+    if (result.isEmpty()) {
+        return QString();
+    }
+
+    QString pathInsidePlace = _url.path().mid(placeUrl.path().length());
+
+    if (!pathInsidePlace.startsWith(QLatin1Char('/'))) {
+        pathInsidePlace.prepend(QLatin1Char('/'));
+    }
+
+    if (pathInsidePlace != QLatin1Char('/')) {
+        // Avoid "500 GiB Internal Hard Drive/foo/bar" when path originates directly from root.
+        const bool isRoot = placeUrl.isLocalFile() && placeUrl.path() == QLatin1Char('/');
+        if (isRoot) {
+            result = pathInsidePlace;
+        } else {
+            result.append(pathInsidePlace);
+        }
+    }
+
+    return result;
+}
+
 void KFilePlacesModelPrivate::initDeviceList()
 {
     Solid::DeviceNotifier *notifier = Solid::DeviceNotifier::instance();
@@ -1331,6 +1367,24 @@ int KFilePlacesModel::hiddenCount() const
     return hidden;
 }
 
+QAction *KFilePlacesModel::openRecentOption(const QModelIndex &index) const
+{
+    const auto bookmark = bookmarkForIndex(index);
+
+    QAction *action = nullptr;
+
+    if (bookmark.url().scheme() == QStringLiteral("recentlyused")) {
+        action = new QAction(QIcon::fromTheme(QStringLiteral("settings-configure")), i18nc("@action:inmenu", "Configure…"), nullptr);
+
+        connect(action, &QAction::triggered, this, []() {
+            auto job = new KIO::CommandLauncherJob(QStringLiteral("systemsettings"), {QStringLiteral("kcm_recentFiles")});
+            job->start();
+        });
+    }
+
+    return action;
+}
+
 QAction *KFilePlacesModel::teardownActionForIndex(const QModelIndex &index) const
 {
     Solid::Device device = deviceForIndex(index);
@@ -1562,6 +1616,7 @@ QAction *KFilePlacesModel::partitionActionForIndex(const QModelIndex &index) con
     connect(action, &QAction::triggered, this, [device] {
         const auto block = device.as<Solid::Block>();
         auto job = new KIO::CommandLauncherJob(partitionManagerPath(), {QStringLiteral("--device"), block->device()});
+        job->setDesktopName(QStringLiteral("org.kde.partitionmanager"));
         job->start();
     });
     return action;
