@@ -20,6 +20,7 @@
 #include <KFileItem>
 #include <KFileItemListProperties>
 #include <KIO/ApplicationLauncherJob>
+#include <KIO/CommandLauncherJob>
 #include <KIO/CopyJob>
 #include <KIO/DndPopupMenuPlugin>
 #include <KIO/FileUndoManager>
@@ -44,7 +45,6 @@
 #include <QMenu>
 #include <QMetaEnum>
 #include <QMimeData>
-#include <QProcess>
 #include <QTimer>
 #include <QWindow>
 
@@ -112,24 +112,6 @@ public:
             m_remoteArkDBusClient = QString::fromUtf8(m_mimeData->data(s_applicationSlashXDashKDEDashArkDashDnDExtractDashService));
             m_remoteArkDBusPath = QString::fromUtf8(m_mimeData->data(s_applicationSlashXDashKDEDashArkDashDnDExtractDashPath));
         }
-
-        if (!(m_flags & KIO::NoPrivilegeExecution)) {
-            m_privilegeExecutionEnabled = true;
-            switch (m_dropAction) {
-            case Qt::CopyAction:
-                m_operationType = Copy;
-                break;
-            case Qt::MoveAction:
-                m_operationType = Move;
-                break;
-            case Qt::LinkAction:
-                m_operationType = Symlink;
-                break;
-            default:
-                m_operationType = Other;
-                break;
-            }
-        }
     }
 
     bool destIsDirectory() const
@@ -302,8 +284,6 @@ void DropJobPrivate::slotStart()
 
 void DropJobPrivate::fillPopupMenu(KIO::DropMenu *popup)
 {
-    Q_Q(DropJob);
-
     const int separatorLength = QCoreApplication::translate("QShortcut", "+").size();
     QString seq = QKeySequence(Qt::ShiftModifier).toString(QKeySequence::NativeText);
     seq.chop(separatorLength); // chop superfluous '+'
@@ -334,7 +314,15 @@ void DropJobPrivate::fillPopupMenu(KIO::DropMenu *popup)
 
     popup->addAction(popupLinkAction);
 
-    addPluginActions(popup, m_itemProps);
+    if (m_dropjobFlags & DropJobFlag::ExcludePluginsActions) {
+        // we must exclude plugins actions so we just call addExtraActions with both parameters as empty lists
+        // to add some final common menu items prepared in that method (usually: last separator and "Cancel" action)
+        QList<QAction *> emptyActionList;
+        popup->addExtraActions(emptyActionList, emptyActionList);
+    } else {
+        // add plugins custom actions to drop popup menu
+        addPluginActions(popup, m_itemProps);
+    }
 }
 
 void DropJobPrivate::addPluginActions(KIO::DropMenu *popup, const KFileItemListProperties &itemProps)
@@ -373,6 +361,7 @@ void DropJob::showMenu(const QPoint &p, QAction *atAction)
     }
 
     for (KIO::DropMenu *menu : std::as_const(d->m_menus)) {
+        menu->ensurePolished();
         if (QWindow *transientParent = d->transientParent()) {
             if (menu->winId()) {
                 menu->windowHandle()->setTransientParent(transientParent);
@@ -423,7 +412,7 @@ void DropJobPrivate::handleCopyToDirectory()
         return;
     }
 
-    if (!m_destItem.isNull() && !m_destItem.isWritable() && (m_flags & KIO::NoPrivilegeExecution)) {
+    if (!m_destItem.isNull() && !m_destItem.isWritable()) {
         slotDropActionDetermined(KIO::ERR_WRITE_ACCESS_DENIED);
         return;
     }
@@ -452,7 +441,12 @@ void DropJobPrivate::handleCopyToDirectory()
         if (mountPoints.isEmpty()) {
             mountPoints = KMountPoint::currentMountPoints();
         }
-        destDevice = mountPoints.findByPath(m_destUrl.path())->mountedFrom();
+        KMountPoint::Ptr destMountPoint = mountPoints.findByPath(m_destUrl.path());
+        if (destMountPoint) {
+            destDevice = destMountPoint->mountedFrom();
+        } else {
+            qCWarning(KIO_WIDGETS) << "Could not determine mount point for destination drop target " << m_destUrl;
+        }
     } else {
         allItemsAreSameDevice = false;
     }
@@ -498,10 +492,17 @@ void DropJobPrivate::handleCopyToDirectory()
             if (mountPoints.isEmpty()) {
                 mountPoints = KMountPoint::currentMountPoints();
             }
-            const QString &sourceDevice = mountPoints.findByPath(url.path())->mountedFrom();
+            QString sourceDevice;
+            KMountPoint::Ptr sourceMountPoint = mountPoints.findByPath(url.path());
+            if (sourceMountPoint) {
+                sourceDevice = sourceMountPoint->mountedFrom();
+            } else {
+                qCWarning(KIO_WIDGETS) << "Could not determine mount point for destination drag source " << url;
+            }
             if (sourceDevice != destDevice && !KFileItem(url).isLink()) {
                 allItemsAreSameDevice = false;
-            } else if (sourceDevice.isEmpty()) {
+            }
+            if (sourceDevice.isEmpty()) {
                 // Sanity check in case we somehow have a local files that we can't get the mount points from.
                 allItemsAreSameDevice = false;
             }
@@ -645,6 +646,7 @@ void DropJobPrivate::slotDropActionDetermined(int error)
         });
 
         if (!(m_dropjobFlags & KIO::ShowMenuManually)) {
+            menu->ensurePolished();
             if (QWindow *parent = transientParent()) {
                 if (menu->winId()) {
                     menu->windowHandle()->setTransientParent(parent);
@@ -744,14 +746,22 @@ void DropJobPrivate::handleDropToDesktopFile()
 void DropJobPrivate::handleDropToExecutable()
 {
     Q_Q(DropJob);
+    const QString destFile = m_destUrl.toLocalFile();
     // Launch executable for each of the files
     QStringList args;
     args.reserve(m_urls.size());
     for (const QUrl &url : std::as_const(m_urls)) {
         args << url.toLocalFile(); // assume local files
     }
-    QProcess::startDetached(m_destUrl.toLocalFile(), args);
-    q->emitResult();
+    auto *job = new KIO::CommandLauncherJob(destFile, args, q);
+    QObject::connect(job, &KJob::result, q, [q, job, destFile] {
+        if (job->error()) {
+            q->setError(KIO::ERR_CANNOT_LAUNCH_PROCESS);
+            q->setErrorText(destFile);
+        }
+        q->emitResult();
+    });
+    job->start();
 }
 
 void DropJob::slotResult(KJob *job)
