@@ -27,12 +27,16 @@
 #include <QScrollBar>
 #include <QToolButton>
 
+#include <KContextualHelpButton>
+#include <KDialogJobUiDelegate>
 #include <KFileUtils>
 #include <KGuiItem>
+#include <KIO/ApplicationLauncherJob>
 #include <KIconLoader>
 #include <KLocalizedString>
 #include <KMessageBox>
 #include <KSeparator>
+#include <KService>
 #include <KSqueezedTextLabel>
 #include <KStandardGuiItem>
 #include <KStringHandler>
@@ -60,20 +64,34 @@ static QLabel *createLabel(QWidget *parent, const QString &text, bool containerT
     return label;
 }
 
+static QString dateLabelText(const KFileItem &item)
+{
+    const QString timeString = item.timeString(KFileItem::ModificationTime);
+    if (timeString.isEmpty()) {
+        return timeString;
+    }
+    return i18n("Date: %1", timeString);
+}
+
 static QLabel *createDateLabel(QWidget *parent, const KFileItem &item)
 {
-    const bool hasDate = item.entry().contains(KIO::UDSEntry::UDS_MODIFICATION_TIME);
-    const QString text = hasDate ? i18n("Date: %1", item.timeString(KFileItem::ModificationTime)) : QString();
-    QLabel *dateLabel = createLabel(parent, text);
+    QLabel *dateLabel = createLabel(parent, dateLabelText(item));
     dateLabel->setAlignment(Qt::AlignLeft | Qt::AlignTop);
     return dateLabel;
 }
 
+static QString sizeLabelText(const KFileItem &item)
+{
+    if (item.entry().contains(KIO::UDSEntry::UDS_SIZE)) {
+        return i18n("Size: %1", KIO::convertSize(item.size()));
+    } else {
+        return {};
+    }
+}
+
 static QLabel *createSizeLabel(QWidget *parent, const KFileItem &item)
 {
-    const bool hasSize = item.entry().contains(KIO::UDSEntry::UDS_SIZE);
-    const QString text = hasSize ? i18n("Size: %1", KIO::convertSize(item.size())) : QString();
-    QLabel *sizeLabel = createLabel(parent, text);
+    QLabel *sizeLabel = createLabel(parent, sizeLabelText(item));
     sizeLabel->setAlignment(Qt::AlignLeft | Qt::AlignBottom);
     return sizeLabel;
 }
@@ -181,12 +199,40 @@ public:
         }
     }
 
+    void maybeShowCompareButton(const KFileItem &fileItem)
+    {
+        if (bCompare && fileItem.currentMimeType().inherits(QStringLiteral("text/plain"))) {
+            bCompare->show();
+        }
+    }
+
+    void updateSrcMetadata(const KFileItem &fileItem)
+    {
+        updateFileMetadata(fileItem, m_srcSizeLabel, m_srcDateLabel);
+    }
+
+    void updateDestMetadata(const KFileItem &fileItem)
+    {
+        updateFileMetadata(fileItem, m_destSizeLabel, m_destDateLabel);
+    }
+
+    void updateFileMetadata(const KFileItem &fileItem, QLabel *sizeLabel, QLabel *dateLabel)
+    {
+        if (const QString sizeText = sizeLabelText(fileItem); !sizeText.isEmpty()) {
+            sizeLabel->setText(sizeText);
+        }
+        if (const QString dateText = dateLabelText(fileItem); !dateText.isEmpty()) {
+            dateLabel->setText(dateText);
+        }
+    }
+
     QPushButton *bCancel = nullptr;
     QPushButton *bRename = nullptr;
     QPushButton *bSkip = nullptr;
     QToolButton *bOverwrite = nullptr;
     QAction *bOverwriteWhenOlder = nullptr;
     QPushButton *bResume = nullptr;
+    QPushButton *bCompare = nullptr;
     QPushButton *bSuggestNewName = nullptr;
     QCheckBox *bApplyAll = nullptr;
     QLineEdit *m_pLineEdit = nullptr;
@@ -196,6 +242,8 @@ public:
     bool m_destPendingPreview = false;
     QLabel *m_srcPreview = nullptr;
     QLabel *m_destPreview = nullptr;
+    QLabel *m_srcSizeLabel = nullptr;
+    QLabel *m_destSizeLabel = nullptr;
     QLabel *m_srcDateLabel = nullptr;
     QLabel *m_destDateLabel = nullptr;
     KFileItem srcItem;
@@ -229,11 +277,11 @@ RenameDialog::RenameDialog(QWidget *parent,
 
     if (_options & RenameDialog_MultipleItems) {
         d->bApplyAll = new QCheckBox(i18n("Appl&y to All"), this);
-        d->bApplyAll->setToolTip((_options & RenameDialog_DestIsDirectory) ? i18n("When this is checked the button pressed will be applied to all "
+        d->bApplyAll->setToolTip((_options & RenameDialog_DestIsDirectory) ? i18n("When this is checked, the button pressed will be applied to all "
                                                                                   "subsequent folder conflicts for the remainder of the current job.\n"
                                                                                   "Unless you press Skip you will still be prompted in case of a "
                                                                                   "conflict with an existing file in the directory.")
-                                                                           : i18n("When this is checked the button pressed will be applied to "
+                                                                           : i18n("When this is checked, the button pressed will be applied to "
                                                                                   "all subsequent conflicts for the remainder of the current job."));
         connect(d->bApplyAll, &QAbstractButton::clicked, this, &RenameDialog::applyAllPressed);
     }
@@ -388,18 +436,19 @@ RenameDialog::RenameDialog(QWidget *parent,
         gridRow++;
 
         // src container (preview, size, date)
-        QLabel *srcSizeLabel = createSizeLabel(this, d->srcItem);
+        d->m_srcSizeLabel = createSizeLabel(this, d->srcItem);
         d->m_srcDateLabel = createDateLabel(this, d->srcItem);
-        QWidget *srcContainer = createContainerWidget(d->m_srcPreview, srcSizeLabel, d->m_srcDateLabel);
+        QWidget *srcContainer = createContainerWidget(d->m_srcPreview, d->m_srcSizeLabel, d->m_srcDateLabel);
         gridLayout->addWidget(srcContainer, gridRow, 0, 1, 2);
 
         // dest container (preview, size, date)
-        QLabel *destSizeLabel = createSizeLabel(this, d->destItem);
+        d->m_destSizeLabel = createSizeLabel(this, d->destItem);
         d->m_destDateLabel = createDateLabel(this, d->destItem);
-        QWidget *destContainer = createContainerWidget(d->m_destPreview, destSizeLabel, d->m_destDateLabel);
+        QWidget *destContainer = createContainerWidget(d->m_destPreview, d->m_destSizeLabel, d->m_destDateLabel);
         gridLayout->addWidget(destContainer, gridRow, 2, 1, 2);
 
         // Verdicts
+        bool offerCompare = false;
         auto *hbox_verdicts = new QHBoxLayout();
         pLayout->addLayout(hbox_verdicts);
         hbox_verdicts->addStretch(1);
@@ -421,6 +470,7 @@ RenameDialog::RenameDialog(QWidget *parent,
                 diff = d->srcItem.size() - d->destItem.size();
                 text = i18n("The source is <b>bigger by %1</b>.", KIO::convertSize(diff));
             }
+            offerCompare = true;
             hbox_verdicts->addWidget(createLabel(this, text));
         }
 
@@ -436,22 +486,22 @@ RenameDialog::RenameDialog(QWidget *parent,
                 break;
             case CompareFilesResult::PartiallyIdentical:
                 text = i18n("The files <b>seem identical</b>.");
+                offerCompare = true;
                 break;
             case CompareFilesResult::Different:
                 text = i18n("The files are <b>different</b>.");
+                offerCompare = true;
                 break;
             }
             QLabel *filesIdenticalLabel = createLabel(this, text);
             if (CompareFilesResult == CompareFilesResult::PartiallyIdentical) {
-                auto *pixmapLabel = new QLabel(this);
-                pixmapLabel->setPixmap(QIcon::fromTheme(QStringLiteral("help-about")).pixmap(QSize(16, 16)));
-                pixmapLabel->setToolTip(
+                auto *helpLabel = new KContextualHelpButton(this);
+                helpLabel->setContextualHelpText(
                     i18n("The files are likely to be identical: they have the same size and their contents are the same at the beginning, middle and end."));
-                pixmapLabel->setCursor(Qt::WhatsThisCursor);
 
                 auto *hbox = new QHBoxLayout();
                 hbox->addWidget(filesIdenticalLabel);
-                hbox->addWidget(pixmapLabel);
+                hbox->addWidget(helpLabel);
                 hbox_verdicts->addLayout(hbox);
             } else {
                 hbox_verdicts->addWidget(filesIdenticalLabel);
@@ -459,6 +509,15 @@ RenameDialog::RenameDialog(QWidget *parent,
         }
         hbox_verdicts->addStretch(1);
 
+        if (offerCompare) {
+            if (auto kompareService = KService::serviceByDesktopName(QStringLiteral("org.kde.kompare"))) {
+                d->bCompare = new QPushButton(QIcon::fromTheme(kompareService->icon()), i18nc("@action:button", "Compare Files"), this);
+                connect(d->bCompare, &QPushButton::clicked, this, &RenameDialog::comparePressed);
+                // It is only shown once the mime type has been actually determined by the preview job.
+                d->bCompare->hide();
+            }
+        }
+
     } else {
         // This is the case where we don't want to allow overwriting, the existing
         // file must be preserved (e.g. when renaming).
@@ -512,6 +571,11 @@ RenameDialog::RenameDialog(QWidget *parent,
     pLayout->addLayout(layout);
 
     layout->setContentsMargins(0, 10, 0, 0); // add some space above the bottom row with buttons
+
+    if (d->bCompare) {
+        layout->addWidget(d->bCompare);
+    }
+
     layout->addStretch(1);
 
     if (d->bApplyAll) {
@@ -695,6 +759,16 @@ void RenameDialog::applyAllPressed()
     }
 }
 
+void RenameDialog::comparePressed()
+{
+    if (auto kompareService = KService::serviceByDesktopName(QStringLiteral("org.kde.kompare"))) {
+        auto *job = new KIO::ApplicationLauncherJob(kompareService);
+        job->setUrls({d->srcItem.url(), d->destItem.url()});
+        job->setUiDelegate(new KDialogJobUiDelegate(KJobUiDelegate::AutoHandlingEnabled, this));
+        job->start();
+    }
+}
+
 void RenameDialog::showSrcIcon(const KFileItem &fileitem)
 {
     // The preview job failed, show a standard file icon.
@@ -703,6 +777,8 @@ void RenameDialog::showSrcIcon(const KFileItem &fileitem)
     const int size = d->m_srcPreview->height();
     const QPixmap pix = QIcon::fromTheme(fileitem.iconName(), QIcon::fromTheme(QStringLiteral("application-octet-stream"))).pixmap(size);
     d->m_srcPreview->setPixmap(pix);
+    d->maybeShowCompareButton(fileitem);
+    d->updateSrcMetadata(fileitem);
 }
 
 void RenameDialog::showDestIcon(const KFileItem &fileitem)
@@ -713,26 +789,28 @@ void RenameDialog::showDestIcon(const KFileItem &fileitem)
     const int size = d->m_destPreview->height();
     const QPixmap pix = QIcon::fromTheme(fileitem.iconName(), QIcon::fromTheme(QStringLiteral("application-octet-stream"))).pixmap(size);
     d->m_destPreview->setPixmap(pix);
+    d->maybeShowCompareButton(fileitem);
+    d->updateDestMetadata(fileitem);
 }
 
 void RenameDialog::showSrcPreview(const KFileItem &fileitem, const QPixmap &pixmap)
 {
-    Q_UNUSED(fileitem);
-
     if (d->m_srcPendingPreview) {
         d->m_srcPreview->setPixmap(pixmap);
         d->m_srcPendingPreview = false;
+        d->maybeShowCompareButton(fileitem);
     }
+    d->updateSrcMetadata(fileitem);
 }
 
 void RenameDialog::showDestPreview(const KFileItem &fileitem, const QPixmap &pixmap)
 {
-    Q_UNUSED(fileitem);
-
     if (d->m_destPendingPreview) {
         d->m_destPreview->setPixmap(pixmap);
         d->m_destPendingPreview = false;
+        d->maybeShowCompareButton(fileitem);
     }
+    d->updateDestMetadata(fileitem);
 }
 
 void RenameDialog::resizePanels()
@@ -748,9 +826,11 @@ void RenameDialog::resizePanels()
     d->m_destDateLabel->setMinimumWidth(minDateWidth);
 
     KIO::PreviewJob *srcJob = KIO::filePreview(KFileItemList{d->srcItem}, QSize(d->m_srcPreview->width() * qreal(0.9), d->m_srcPreview->height()));
+    srcJob->setDevicePixelRatio(devicePixelRatioF());
     srcJob->setScaleType(KIO::PreviewJob::Unscaled);
 
     KIO::PreviewJob *destJob = KIO::filePreview(KFileItemList{d->destItem}, QSize(d->m_destPreview->width() * qreal(0.9), d->m_destPreview->height()));
+    destJob->setDevicePixelRatio(devicePixelRatioF());
     destJob->setScaleType(KIO::PreviewJob::Unscaled);
 
     connect(srcJob, &PreviewJob::gotPreview, this, &RenameDialog::showSrcPreview);
