@@ -14,7 +14,6 @@
 #include <kio/listjob.h>
 #include <kio/mimetypejob.h>
 #include <kio/mkdirjob.h>
-#include <kio/simplejob.h>
 #include <kio/statjob.h>
 #include <kio/transferjob.h>
 #include <kprotocolinfo.h>
@@ -246,7 +245,7 @@ void KioWorkerTest::startJob()
     }
 
     case Stat:
-        job = KIO::stat(src);
+        job = KIO::stat(src, KIO::StatJob::StatSide::SourceSide, KIO::StatDefaultDetails | KIO::StatSubVolId | KIO::StatMountId, KIO::DefaultFlags);
         break;
 
     case Get: {
@@ -382,6 +381,12 @@ void KioWorkerTest::printUDSEntry(const KIO::UDSEntry &entry)
             timestamp = QDateTime::fromSecsSinceEpoch(entry.numberValue(*it));
             qDebug() << "AccessTime: " << QLocale().toString(timestamp, QLocale::ShortFormat);
             break;
+        case KIO::UDSEntry::UDS_SUBVOL_ID:
+            qDebug() << "Subvolume id : " << entry.numberValue(*it);
+            break;
+        case KIO::UDSEntry::UDS_MOUNT_ID:
+            qDebug() << "Mount id : " << entry.numberValue(*it);
+            break;
         }
     }
 }
